@@ -2,9 +2,6 @@
 #include "distr.h"
 #include "utils.h"
 
-// [[Rcpp::depends(RcppArmadillo)]]
-#include <RcppArmadilloExtensions/sample.h>
-
 #ifndef CCODE
 	using Rcpp::Rcout;
 	#define BOOST_DISABLE_ASSERTS
@@ -18,91 +15,51 @@
    #define omp_get_thread_num() 0
 #endif
 
-//defined in global.h
-//extern std::vector<std::mt19937_64> rng;
-
-namespace Distributions{
+using namespace Rcpp;
 
+    // [[Rcpp::export]]
 	double randU01()
 	{
-		//std::uniform_real_distribution<> distr(0, std::nextafter(1, std::numeric_limits<double>::max())); // init U(0,1)
-		//return distr(rng[omp_get_thread_num()]);
-        return R::runif( 0., 1. );
+		return R::runif( 0., 1. );
 	}
 
+    // [[Rcpp::export]]
 	double randLogU01()
 	{
-		//std::uniform_real_distribution<> distr(0, std::nextafter(1, std::numeric_limits<double>::max())); // init U(0,1)
-		//return log(distr(rng[omp_get_thread_num()]));
-        return log( R::runif( 0., 1. ) );
+		return log( R::runif( 0., 1. ) );
 	}
 
+    // [[Rcpp::export]]
 	int randIntUniform(const int a,const int b)
 	{
-		//std::uniform_int_distribution<> distr(a, b); // init the discrete uniform
-		//return distr(rng[omp_get_thread_num()]);
-        /*Rcpp::IntegerVector pool = Rcpp::seq(a, b);
-        std::random_shuffle(pool.begin(), pool.end());
-        return pool[0];*/
-        /*arma::vec probs = arma::ones<arma::vec>( b-a+1 )/(double)(b-a+1);
-        arma::ivec idTmp( probs.n_elem );
-        R::rmultinom( 1, probs.begin(), b-a+1, idTmp.begin() );
-        return a-1+arma::conv_to<int>::from(arma::find(idTmp == 1));*/
-        arma::vec res = Rcpp::RcppArmadillo::sample(arma::linspace(a, b, b-a+1), 1, true, arma::ones( b-a+1 )/(double)(b-a+1));
-        return arma::conv_to<int>::from(res);
-	}
-
-	arma::ivec randIntUniform(const unsigned int n, const int a,const int b)
-	{
-		arma::ivec res(n);
-		/*std::uniform_int_distribution<> distr(a, b); // init the discrete uniform
-		for(unsigned int i=0; i<n; ++i)
-		{
-			res(i) = distr(rng[omp_get_thread_num()]);
-		}
-        */
-        /*arma::vec probs = arma::ones<arma::vec>( b-a+1 )/(double)(b-a+1);
-        arma::ivec idTmp( probs.n_elem );
-        for(unsigned i=0; i<n; i++){
-          R::rmultinom( 1, probs.begin(), b-a+1, idTmp.begin() );
-          res(i) = a-1+arma::conv_to<int>::from(arma::find(idTmp == 1));
-        }*/
-        //Rcpp::IntegerVector pool = Rcpp::seq(a, b);
-        for(unsigned i=0; i<n; i++){
-          res(i) = arma::conv_to<int>::from( Rcpp::RcppArmadillo::sample(arma::linspace(a, b, b-a+1), 1, true, arma::ones( b-a+1 )/(double)(b-a+1)) );
-            //std::random_shuffle(pool.begin(), pool.end());
-            //res(i) = pool[0];
-        }
-        return res;
+		return ceil( R::runif( a-1, b ) );
 	}
 
+    // [[Rcpp::export]]
 	double randExponential(const double lambda)
 	{
-		//std::exponential_distribution<> distr(lambda);
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rexp( lambda );
+		return R::rexp( lambda );
 	}
 
-	arma::vec randExponential(const unsigned int n, const double lambda)
+    // [[Rcpp::export]]
+	arma::vec randVecExponential(const unsigned int n, const double lambda)
 	{
 		arma::vec res(n);
-		//std::exponential_distribution<> distr(lambda);
 		for(unsigned int i=0; i<n; ++i)
 		{
-			//res(i) = distr(rng[omp_get_thread_num()]);
-            res(i) = R::rexp( lambda );
+			res(i) = R::rexp( lambda );
 		}
 		return res;
 	}
 
+    // [[Rcpp::export]]
 	unsigned int randBinomial(const unsigned int n, const double p) // slow but safe (CARE, n here is the binomial parameters, return value is always ONE integer)
 	{
-		//std::binomial_distribution<> distr(n, p);
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rbinom( n, p );
+		return R::rbinom( n, p );
         
 	}
 
+    // [[Rcpp::export]]
 	arma::uvec randMultinomial(unsigned int n, const arma::vec prob)
 	{
 
@@ -130,126 +87,83 @@ namespace Distributions{
 
 	}
 
-
+    // [[Rcpp::export]]
 	double randNormal(const double m=0., const double sigmaSquare=1.) // random normal interface, parameters mean and variance
 	{
 		if( sigmaSquare< 0 )
-			throw negativeParameters();
+			throw Distributions::negativeParameters();
 
-    	//std::normal_distribution<> distr(m,sqrt(sigmaSquare));
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rnorm( m, sigmaSquare );
+    	return R::rnorm( m, sigmaSquare );
 	}
 
-	arma::vec randNormal(const unsigned int n, const double m=0., const double sigmaSquare=1.) // n-sample normal, parameters mean and variance
+    // [[Rcpp::export]]
+	arma::vec randVecNormal(const unsigned int n, const double m=0., const double sigmaSquare=1.) // n-sample normal, parameters mean and variance
 	{
 
 		if( sigmaSquare< 0 )
-			throw negativeParameters();
+			throw Distributions::negativeParameters();
         
     	arma::vec res(n);
-    	//std::normal_distribution<> distr(m,sqrt(sigmaSquare));
     	for(unsigned int i=0; i<n; ++i)
 		{
-			//res(i) = distr(rng[omp_get_thread_num()]);
-            res(i) = R::rnorm( m, sigmaSquare );
+			res(i) = R::rnorm( m, sigmaSquare );
 		}
 		return res;
 	}
 
-	arma::vec randMvNormal(const arma::vec &m, const arma::mat &Sigma) // random normal interface to arma::randn
-	{
-		unsigned int d = m.n_elem;
-		//check
-		if(Sigma.n_rows != d || Sigma.n_cols != d )
-		{
-			Rcout << " Dimension not matching in the multivariate normal sampler";
-			throw dimensionsNotMatching();
-		}
-
-			arma::mat A;
-			arma::vec eigval;
-			arma::mat eigvec;
-			arma::rowvec res;
-
-			if( arma::chol(A,Sigma) )
-			{
-				res = randNormal(d).t() * A ;
-			}
-			else
-			{
-				if( eig_sym(eigval, eigvec, Sigma) )
-				{
-					res = (eigvec * arma::diagmat(arma::sqrt(eigval)) * randNormal(d)).t();
-				}else{
-					Rcout << "randMvNorm failing because of singular Sigma matrix" << '\n';
-					throw negativeDefiniteParameters();
-				}
-			}
-
-		return res.t() + m;
-	}
-
+    // [[Rcpp::export]]
 	double randT(const double nu)
 	{
-    	//std::student_t_distribution<double> distr(nu);
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rt( nu );
+    	return R::rt( nu );
 	}
 
-	arma::vec randT(const unsigned int n, const double nu)
+    // [[Rcpp::export]]
+	arma::vec randVecT(const unsigned int n, const double nu)
 	{
     	arma::vec res(n);
-    	//std::student_t_distribution<double> distr(nu);
     	for(unsigned int i=0; i<n; ++i)
 		{
-			//res(i) = distr(rng[omp_get_thread_num()]);
-            res(i) = R::rt( nu );
+			res(i) = R::rt( nu );
 		}
 		return res;
 	}
 
+    // [[Rcpp::export]]
 	arma::vec randMvT(const double &nu, const arma::vec &m, const arma::mat &Sigma)
 	{
 		unsigned int d = m.n_elem;
-		arma::rowvec res = randT(d,nu).t() * arma::chol(Sigma);
+		arma::rowvec res = randVecT(d,nu).t() * arma::chol(Sigma);
 
 		return res.t() + m;
 	}
 
-
-
+    // [[Rcpp::export]]
 	double randGamma(double shape, double scale)   // shape scale parametrisation
 	{
 		//check
 		if(shape <= 0 || scale <= 0 )
 		{
-			throw negativeParameters(); // THROW EXCPTION
+			throw Distributions::negativeParameters(); // THROW EXCPTION
 		}
 
-		//std::gamma_distribution<> distr(shape,scale);
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rgamma( shape, scale );
+		return R::rgamma( shape, scale );
 	}
 
-
+    // [[Rcpp::export]]
 	double randIGamma(double shape, double scale)
 	{
 		//check
 		if(shape <= 0 || scale <= 0 )
 		{
 			Rcout << " Negative parameter in the gamma sampler " << '\n';
-			throw negativeParameters(); // THROW EXCPTION
+			throw Distributions::negativeParameters(); // THROW EXCPTION
 		}
 
-		//std::gamma_distribution<> distr(shape,1./scale);
-		//return  ( 1./distr(rng[omp_get_thread_num()]) );
-        return 1./R::rgamma(shape, 1./scale);
+		return 1./R::rgamma(shape, 1./scale);
         //return 1./Rcpp::rgamma(1, shape, 1./scale)[0];
 	}
 
-
-
+    // [[Rcpp::export]]
 	arma::mat randWishart(double df, const arma::mat& S)   // unsigned int df is obsolete, I see no reason to keep it
 	{
 		// Dimension of returned wishart
@@ -272,8 +186,7 @@ namespace Distributions{
 		// Fill the lower matrix with random normals
 		for(unsigned int j = 0; j < m; j++){
 			for(unsigned int i = j+1; i < m; i++){
-		  		//Z(i,j) = normal01(rng[omp_get_thread_num()]);
-                Z(i,j) = R::rnorm(0.,1.);
+		  		Z(i,j) = R::rnorm(0.,1.);
 			}
 		}
 
@@ -284,40 +197,69 @@ namespace Distributions{
 		return C.t()*C;
 	}
 
-
-	arma::mat randIWishart(double df, const arma::mat& S)
-	{
-
-  		return arma::inv_sympd( randWishart(df,S.i()) );   // return the inverse of the correspondent Wishart variate ... is this even fast enough?
-	}
-
-
-	arma::mat randMN(const arma::mat &M, const arma::mat &rowCov, const arma::mat &colCov)
-	{
-		arma::mat C = arma::chol( arma::kron(colCov,rowCov) );
-		arma::mat z = randNormal( (unsigned int)(M.n_cols * M.n_rows) ).t() * C;
-		z.reshape( arma::size(M) );
-		return (z + M);
-	}
-
-
+    // [[Rcpp::export]]
 	double randBeta(double a, double b)
 	{
-		//double num = randGamma(a,1.);
-		//double den = randGamma(b,1.) + num;
-
-		//return num/den;
-        return R::rbeta( a, b );
+		return R::rbeta( a, b );
 	}
 
-
+    // [[Rcpp::export]]
 	unsigned int randBernoulli(double pi)
 	{
-		//std::bernoulli_distribution distr(pi);
-		//return distr(rng[omp_get_thread_num()]);
-        return R::rbinom( 1, pi );
+		return R::rbinom( 1, pi );
 	}
 
+
+namespace Distributions{
+
+    arma::mat randIWishart(double df, const arma::mat& S)
+    {
+
+          return arma::inv_sympd( randWishart(df,S.i()) );   // return the inverse of the correspondent Wishart variate ... is this even fast enough?
+    }
+
+    arma::vec randMvNormal(const arma::vec &m, const arma::mat &Sigma) // random normal interface to arma::randn
+    {
+        unsigned int d = m.n_elem;
+        //check
+        if(Sigma.n_rows != d || Sigma.n_cols != d )
+        {
+            Rcout << " Dimension not matching in the multivariate normal sampler";
+            throw dimensionsNotMatching();
+        }
+
+            arma::mat A;
+            arma::vec eigval;
+            arma::mat eigvec;
+            arma::rowvec res;
+
+            if( arma::chol(A,Sigma) )
+            {
+                res = randVecNormal(d).t() * A ;
+            }
+            else
+            {
+                if( eig_sym(eigval, eigvec, Sigma) )
+                {
+                    res = (eigvec * arma::diagmat(arma::sqrt(eigval)) * randVecNormal(d)).t();
+                }else{
+                    Rcout << "randMvNorm failing because of singular Sigma matrix" << '\n';
+                    throw negativeDefiniteParameters();
+                }
+            }
+
+        return res.t() + m;
+    }
+
+    arma::mat randMN(const arma::mat &M, const arma::mat &rowCov, const arma::mat &colCov)
+    {
+        arma::mat C = arma::chol( arma::kron(colCov,rowCov) );
+        arma::mat z = randVecNormal( (unsigned int)(M.n_cols * M.n_rows) ).t() * C;
+        z.reshape( arma::size(M) );
+        return (z + M);
+    }
+
+
 	double randTruncNorm(double m, double sd,double lower, double upper) // Naive, but it'll do for now -- notice now parameters are mean and standard deviation!
 	{
 		double ret = randNormal(m,sd);
@@ -408,7 +350,7 @@ namespace Distributions{
 	    const arma::uvec& population // population to draw from
 	) // sample is a zero-offset indices to selected items, output is the subsampled population.
 	{
-	    arma::vec score = randExponential(populationSize,1.)/weights;
+	    arma::vec score = randVecExponential(populationSize,1.)/weights;
 	    arma::uvec result = population( (arma::sort_index(weights,"ascend")) );
 
 	    return result.subvec(0,sampleSize-1);
@@ -444,7 +386,7 @@ namespace Distributions{
 	) // sample is a zero-offset indices to selected items, output is the subsampled population.
 	{
 		// note I can do everything in the log scale as the ordering won't change!
-	    arma::vec score = randExponential(populationSize,1.) - weights;
+	    arma::vec score = randVecExponential(populationSize,1.) - weights;
 	    arma::uvec result = arma::sort_index(score,"ascend");
 
 	    return result.subvec(0,sampleSize-1);
@@ -458,7 +400,7 @@ namespace Distributions{
 	) // sample is a zero-offset indices to selected items, output is the subsampled population.
 	{
 		// note I can do everything in the log scale as the ordering won't change!
-	    arma::vec score = randExponential(populationSize,1.);
+	    arma::vec score = randVecExponential(populationSize,1.);
 	    arma::uvec result = arma::sort_index(score,"ascend");
         
 	    return result.subvec(0,sampleSize-1);
